#pragma once
#ifndef _CPISP_FUNC_H
#define  _CPISP_FUNC_H
#include "../../Tool_Common/CommonDefine/CommonDefine.h"
#include "../../CpComm/Comm_Export/CpCommExport.h"
#include "../../Tool_Common/CpRsString.h"
#include "../../Tool_Common/RsCommon/RegDef.h"
#include "../../Tool_Common/CpRsFuncBase.h"
#include "CpIspSetting.h"
#include "CpFlashDat.h"
#include "CpIspProgress.h"



CONST BYTE UC_SLAVE_DDCCI_MODE = 0x6E;
CONST BYTE UC_SLAVE_DEBUG_MODE = 0x6A;
CONST BYTE UC_SLAVE_QISDA_FACTORY = 0x7C;
CONST BYTE UC_SLAVE_UNKNOWN = 0xFF;


#define UC_VERSION_DATA_LENGTH 26
#define UC_BOOT_BANK 0
#define UC_ROM_BANK 1
#define UL_MAX_SERIALBANK_LENGTH 65536 
#define UL_RESERVE_ITEM_COUNT 8
#define UC_FLASH_BOOT_FLAG_LENGTH 2
#define UC_ERASE_BANK_CRC 0xDE
#define UC_ERASE_SECTOR_CRC 0x09
// For flash partition page
#define UL_BOOT_FLAG_COUNT 8
#define UL_ROM_FLAG_COUNT 8
#define UL_PAGE_LEN 256
#define UL_PAGE_COUNT_PER_BANK 256
#define UC_SLAVE_ADDR 0x94
#define MAX_DIR_LENGTH 256

#define UL_TARGET_IC_USE_BANK_PAGE_COUNT 249
#define UL_TARGET_IC_REMAIN_BANK_PAGE_COUNT (256 - UL_TARGET_IC_USE_BANK_PAGE_COUNT)
#define UL_SECTOR_LEN (UL_PAGE_LEN * 16)
#define UL_MAX_BANK_COUNT 256

#define STR_FLASH_NAME_DEFAULT "Default"
#define STR_FLASH_NAME_PM25LD020 "PM25LD020"
#define STR_FLASH_NAME_GD25M21B "GD25M21_B"
#define STR_FLASH_NAME_GD25M41B "GD25M41_B"
#define STR_FLASH_NAME_PM25LX040 "PM25LX040"
#define STR_FLASH_NAME_PM25LQ080 "PM25LQ080"
#define STR_FLASH_NAME_PM25LQ020 "PM25LQ020"
#define STR_FLASH_NAME_PM25LQ016 "PM25LQ016"
#define STR_FLASH_NAME_W25Q05CLWI "W25Q05CLWI"
#define STR_FLASH_NAME_W25Q20CL "W25Q20CL"
#define STR_FLASH_NAME_IS25LQ040 "IS25LQ040"

// For erase setting, when auto sector index fail, put the invalid sector index
#define SECTOR_INDEX_INVALID 0xFF
#define N_PM25LD020_SAFE_GUARD_REG_COUNT 10
#define N_GD25M21B_SECURITY_REG_COUNT 8
#define N_PM25LQ_SECURITY_REG_COUNT   64

#define IF_ERROR_THEN_RETURN(enumError) {if(_ERROR_SUCCESS != enumError) return enumError;}

#define IS_Digital_Sign_Verify(FuncType) ((FuncType == _ENUM_FUNC_ISP_NORMAL_HW_SIGN) || (FuncType == _ENUM_FUNC_ISP_NORMAL_SW_SIGN) \
|| (FuncType == _ENUM_FUNC_ISP_FW_PARTITION_HW_SIGN) || (FuncType == _ENUM_FUNC_ISP_FW_PARTITION_SW_SIGN) || (FuncType == _ENUM_FUNC_ISP_DIFFER_BANK_HW_SIGN) \
|| (FuncType == _ENUM_FUNC_ISP_DIFFER_BANK_SW_SIGN) || (FuncType == _ENUM_FUNC_ISP_COPY_BANK_HW_SIGN) || (FuncType == _ENUM_FUNC_ISP_COPY_BANK_SW_SIGN))

#define IS_Digital_HW_Sign_Verify(FuncType) ((FuncType == _ENUM_FUNC_ISP_NORMAL_HW_SIGN) \
|| (FuncType == _ENUM_FUNC_ISP_COPY_BANK_HW_SIGN) || (FuncType == _ENUM_FUNC_ISP_DIFFER_BANK_HW_SIGN))

#define FUNCTYPE_ENABLE_CHIP_ERASE(FuncType) ((FuncType == _ENUM_FUNC_ISP_NORMAL ) || (FuncType == _ENUM_FUNC_ISP_NORMAL_HW_SIGN) || (FuncType == _ENUM_FUNC_ISP_NORMAL_SW_SIGN) \
|| (FuncType == _ENUM_FUNC_ERASE ))

#define IS_BOOT_FUNC_TYPE(FuncType) ((FuncType == _ENUM_FUNC_ISP_NORMAL) || (FuncType == _ENUM_FUNC_ISP_NORMAL_HW_SIGN) \
|| (FuncType == _ENUM_FUNC_ISP_NORMAL_SW_SIGN))

#define IS_USER_FUNC_TYPE(FuncType) ((FuncType == _ENUM_FUNC_ISP_FW_PARTITION) || (FuncType == _ENUM_FUNC_ISP_FW_PARTITION_HW_SIGN)\
|| (FuncType == _ENUM_FUNC_ISP_FW_PARTITION_SW_SIGN) || (FuncType == _ENUM_FUNC_ISP_DIFFER_BANK) || (FuncType == _ENUM_FUNC_ISP_DIFFER_BANK_HW_SIGN)\
|| (FuncType == _ENUM_FUNC_ISP_DIFFER_BANK_SW_SIGN) || (FuncType == _ENUM_FUNC_ISP_COPY_BANK) || (FuncType == _ENUM_FUNC_ISP_COPY_BANK_HW_SIGN)\
|| (FuncType == _ENUM_FUNC_ISP_COPY_BANK_SW_SIGN))

#define IS_CLEAR_EXTRA_FLAG_FUNC_TYPE(FuncType) ((FuncType == _ENUM_FUNC_ISP_COPY_BANK) || (FuncType == _ENUM_FUNC_ISP_COPY_BANK_SW_SIGN))

#define IS_COPY_BANK_FUNC_TYPE(FuncType) ((FuncType == _ENUM_FUNC_ISP_COPY_BANK) || (FuncType == _ENUM_FUNC_ISP_COPY_BANK_HW_SIGN)\
|| (FuncType == _ENUM_FUNC_ISP_COPY_BANK_SW_SIGN))

typedef struct STRUCT_ROM_FLAG_SETTING
{
	// Address and flag data
	BYTE ucRomBank;
	DWORD ulAddr;
	/*---The actual len---*/
	/*---find the first no 0xFF flag from ---*/
	/*---the ucszFlag[UL_ROM_FLAG_COUNT - 1] to ucszFlag[0]---*/
	BYTE ucszFlag[UL_ROM_FLAG_COUNT];
}STRUCT_ROM_FLAG_SETTING;

typedef enum STRUCT_ENUM_SIGN_STATUS
{
	_ENUM_SIGN_PASS = 0,
	_ENUM_SIGN_FAIL,
	_ENUM_SIGN_INVALID,
	_ENUM_SIGN_ERROR
}STRUCT_ENUM_SIGN_STATUS;

typedef enum
{
	_ENUM_FLASH_MCM = 0,
	_ENUM_FLASH_EXT,
}ENUM_TRANSLATOR_FLASH_MODE;

// Do not change the sequence
typedef enum
{
	_ENUM_CMD_SLEEP = 0x00,
	_ENUM_CMD_SCALERRUN,
	_ENUM_CMD_SCALERSTOP,
	_ENUM_CMD_WRITEREG,
	_ENUM_CMD_WRITEREGBIT,
	_ENUM_CMD_READREG, // 0x05
	_ENUM_CMD_WRITEMUCREG,
	_ENUM_CMD_READMCUREG,
	_ENUM_CMD_I2CWRITEBYTE,
	_ENUM_CMD_I2CREADBYTE,
	_ENUM_CMD_I2C2SUBWRITEBYTE, // 0x0A
	_ENUM_CMD_I2C2SUBREADBYTE,
	_ENUM_CMD_I2CCURRENTREAD,
	_ENUM_CMD_POLLING_MCUREG,
	_ENUM_CMD_WRITEPORTREGBIT,
	_ENUM_CMD_WRITEMCUREGBIT,

	_ENUM_CMD_WRITEREG_EX, // 0x10
	_ENUM_CMD_WRITEREGBIT_EX,
	_ENUM_CMD_READREG_EX,
	_ENUM_CMD_WRITEPORTREGBIT_EX,
	_ENUM_CMD_WRITEREGS_EX,
	_ENUM_CMD_READREGS_EX,//0x15
	_ENUM_CMD_WRITEREGS,
	_ENUM_CMD_READREGS,
	_ENUM_CMD_WRITEMCUREGS,
	_ENUM_CMD_READMCUREGS,
	_ENUM_CMD_WRITE32REG,//0x1A

}STRUCT_ENUM_CMD_ID;

typedef struct
{
	INT nMajor;
	INT nMinor;
	INT nBuild;
	INT nRevision;
}STRUCT_FW_VERSION;

typedef struct STRUCT_READ_FW_VERSION
{
	WORD usProjectID;
	BYTE ucFWVer;
	BYTE ucFWSubVer;
}STRUCT_READ_FW_VERSION;

extern CCpRsComm *g_pcpRsComm;
void LogRecord(CCpRsString strLog);

/////////////////////////////////Overwrite in windows and linux/////////////////////////////////////////
void ShowMsg(CCpRsString str);
DWORD CpRsGetTickCount();
string GetFlashDatPathPlusName(string strPath);
string GetSettingFilePath(string strCurDir);
/////////////////////////////////Overwrite in windows and linux/////////////////////////////////////////

STRUCT_ENUM_ERROR_TYPE SetMcuRegDefaultValue();
STRUCT_ENUM_ERROR_TYPE SetDDCDebounce();
STRUCT_ENUM_ERROR_TYPE SetDDC0DDC1ADCDDCDebounce();
STRUCT_ENUM_ERROR_TYPE SetDDC2Debounce();
STRUCT_ENUM_ERROR_TYPE SetDDC3Debounce();
STRUCT_ENUM_ERROR_TYPE SetDDC4Debounce();
STRUCT_ENUM_ERROR_TYPE SetDDC5Debounce();
STRUCT_ENUM_ERROR_TYPE SetWPPin();

void ResetMcu();
void ResetMcuAsSetting();
STRUCT_ENUM_ERROR_TYPE EnterIspMode(WORD usRetryTimes);
STRUCT_ENUM_ERROR_TYPE WaitForBitChange(BYTE ucRegister, BYTE ucAnd, BYTE ucTarget, INT nTryCount);
STRUCT_ENUM_ERROR_TYPE RunCommand(BYTE* pucCmd, DWORD ulCmdLength, CCpRsComm* pcpRsComm, BYTE* pucOutput, DWORD *pulOutPutLen = NULL);
STRUCT_ENUM_ERROR_TYPE ReadFlashSR(BYTE& ucSRValue, BYTE ucOPCode);
STRUCT_ENUM_ERROR_TYPE WriteFlashSR(BYTE *pucSRValue, CCpFlash* pcpFlash);
STRUCT_ENUM_ERROR_TYPE ReadCRC(DWORD ulBankIndex, WORD usStartAddr, DWORD ulCount, BYTE* pucCRC);
BYTE CalculateByteCRC(LPBYTE pucData, DWORD ulLength);
STRUCT_ENUM_ERROR_TYPE ChipErase(CCpFlash *pcpFlash);
STRUCT_ENUM_ERROR_TYPE BankErase(DWORD ulBankIndex, CCpFlash *pcpFlash);
STRUCT_ENUM_ERROR_TYPE SectorErase(DWORD ulBankIndex, DWORD ulSectorIndex, CCpFlash* pcpFlash);
STRUCT_ENUM_ERROR_TYPE ReadFlashData(WORD usBankAddr, DWORD ulStartAddr, DWORD ulCount, BYTE* pucBuff);
STRUCT_ENUM_ERROR_TYPE ClearBootFlag(DWORD ulAddr, DWORD ulFlagLenth, BYTE *pucFlag, CCpFlash *pcpFlash);
STRUCT_ENUM_ERROR_TYPE WriteDataToFlashByTypeCUsb(DWORD ulDesAddr, DWORD ulProLen, BYTE *pucBuff, CCpFlash *pcpFlash);
STRUCT_ENUM_ERROR_TYPE WriteDataToFlash(DWORD ulDesAddr, DWORD ulProLen, BYTE *pucBuff, CCpFlash *pcpFlash);
STRUCT_ENUM_ERROR_TYPE WriteBootFlag(DWORD ulAddr, DWORD ulFlagLength, BYTE *pucFlag, CCpFlash *pcpFlash);
void LoadFlashData(std::map<string, CCpFlash> &mapFlashInfo, CCpRsString cpstrPath);
STRUCT_ENUM_ERROR_TYPE GetFlashType(std::map<string, CCpFlash> &mapFlashInfo, CCpFlash &cpFlash);
STRUCT_ENUM_ERROR_TYPE SetI2CSpeed(WORD usSpeed);
string GetLogPathPlusName(string strLog);
STRUCT_ENUM_ERROR_TYPE GD25M21BReadSecurityShieldReg(BYTE *pucRead);
STRUCT_ENUM_ERROR_TYPE GD25M21BProgramSecurityShieldReg(BYTE *pucRead);
STRUCT_ENUM_ERROR_TYPE RunSafeGuardCmd(CCpFlash &cpFlash, BOOL bEnableWP);
STRUCT_ENUM_ERROR_TYPE LoadFWBank(string strFwPath, DWORD ulBankNum, BYTE *pucBuff);
DWORD GetValidLength(BYTE *pucBuff, DWORD ulBuffLen);
STRUCT_ENUM_ERROR_TYPE EnableSoftWareWP(CCpFlash &cpFlash, BYTE *pucszReadGD25M21BSecurityRegs);
STRUCT_ENUM_ERROR_TYPE GetFinalCode(DWORD ulBankIndex, STRUCT_FW_BANK_SETTING stFwBankSetting, BYTE *pucBuff, DWORD *pulProlen, BOOL *pbSectorErase, DWORD *pulSectorEraseCount, std::map<DWORD, STRUCT_RESERVE_ITEM> mapReserveItem);
STRUCT_ENUM_ERROR_TYPE DisableSoftWareWP(CCpFlash &cpFlash, BYTE *pucszReadGD25M21BSecurityRegs);

STRUCT_ENUM_ERROR_TYPE WriteExtraBootFlag(CCpFlash &cpFlash);
STRUCT_ENUM_ERROR_TYPE EraseOperation(CCpFlash &cpFlash, BYTE *pucszReadGD25M21BSecurityRegs, std::map<DWORD, STRUCT_RESERVE_ITEM> mapReservedItem);
STRUCT_ENUM_ERROR_TYPE CheckRomFlagOperation(BOOL bClearRomFlag, CCpFlash &cpFlash, STRUCT_ROM_FLAG_SETTING &stRomFlag);
STRUCT_ENUM_ERROR_TYPE UserFlagOperation(BOOL bClearUserFlag, CCpFlash &cpFlash);
STRUCT_ENUM_ERROR_TYPE CheckRsComm(STRUCT_CP_ISP_COMMAND_SETTING &stCmdSetting);
STRUCT_ENUM_ERROR_TYPE GetFWVersion(STRUCT_READ_FW_VERSION *pstReadFWVer);

#endif
